#pragma once

/* Genode includes */
#include <util/xml_node.h>

/* local includes */
#include "context.h"
#include "types.h"


namespace State_machine
{
	using namespace Genode;

	class Action;
	class Action_factory;
}


class State_machine::Action
{
	private:

		Context         &_context;
		Xml_node  const  _config;

	protected:

		Context         &context()      { return _context; }
		Xml_node  const &config() const { return _config; }

	public:

		virtual void perform() = 0;

		Action(Context &context, Xml_node const &config) :
			_context { context },
			_config  { config }
		{ }

		virtual ~Action() = default;
};


class State_machine::Action_factory
{
	public:
		virtual Action &create(Context &context, Xml_node const &config) = 0;

		virtual ~Action_factory() = default;
};
