#pragma once

/* Genode includes */
#include <base/signal.h>
#include <util/xml_node.h>

/* local includes */
#include "context.h"
#include "types.h"


namespace State_machine
{
	using namespace Genode;

	class Condition;
	class Condition_factory;
}


class State_machine::Condition
{
	private:

		Context                     &_context;
		Xml_node const               _config;
		bool                         _result { false };
		Signal_context_capability    _sigh { };

	protected:

		Context        &context()      { return _context; }
		Xml_node const &config() const { return _config; }

		virtual bool evaluate() = 0;

		void notify_sigh()
		{
			if (_sigh.valid())
				Signal_transmitter(_sigh).submit();
		}

	public:

		bool result() const { return _result; }

		void compute() {
			_result = evaluate();
		}

		void sigh(Signal_context_capability sigh)
		{
			_sigh = sigh;
		}

		Condition(Context &context, Xml_node const &config) :
			_context  { context },
			_config   { config }
		{ }

		virtual ~Condition() = default;
};


class State_machine::Condition_factory
{
	public:
		virtual Condition &create(Context &context, Xml_node const &config) = 0;

		virtual ~Condition_factory() = default;
};
