#pragma once

/* Genode includes */
#include <base/allocator.h>
#include <base/env.h>
#include <base/heap.h>

/* local includes */
#include "types.h"


namespace State_machine
{
	using namespace Genode;

	class Context;

	class Action;
	class Condition;
	class Provider;
}


class State_machine::Context
{
	private:

		Env   &_env;
		Heap  &_heap;
		bool   _verbose;

	public:

		Env          &env()           { return _env; }
		Allocator    &alloc()         { return _heap; }
		bool          verbose() const { return _verbose; }

		/* interface to load/stat plugins */
		virtual Condition &create_condition(Genode::Xml_node const &node) = 0;
		virtual Provider  &create_provider (Genode::Xml_node const &node) = 0;
		virtual Action    &create_action   (Genode::Xml_node const &node) = 0;

		/* interface to manage variables */
		virtual Variable_value &get_variable          (Variable_name const &name) const = 0;
		virtual void            set_variable          (Variable_name const &name, Variable_value const &value) = 0;
		virtual void            register_variable_sigh(Variable_name const &name, Signal_context_capability sigh) = 0;

		/* interface to manage progress of the state machine */
		virtual Signal_context_capability signal_handler_cap() const = 0;
		virtual void                      set_progressed() = 0;

		Context(Env &env, Heap &heap, bool verbose) :
			_env     { env },
			_heap    { heap },
			_verbose { verbose }
		{ }

		virtual ~Context() = default;
};
